//
//  PopupMenu.m
//  CutTheVerlet
//
//  Created by Fahim Farook on 7/4/13.
//  Copyright (c) 2013 OpenTheJob. All rights reserved.
//

#import "PopupMenu.h"
#import "RopeSprite.h"

@interface PopupMenu () {
    CCSprite* background;
    CCMenu* menu;
    CCMenuItem* ropeItem;
    CGPoint tapPosition;
    BOOL isEnabled;
}

@end

@implementation PopupMenu

-(id)initWithParent:(CCNode*) parent {
    self = [super init];
    if (self) {
        CCSprite* pineappleSprite = [CCSprite spriteWithFile:@"pineappleitem.png"];
        CCSprite* pineappleSpriteSelected = [CCSprite spriteWithFile:@"pineappleitem.png"];
        pineappleSpriteSelected.color = ccc3(100, 0, 0);
        CCMenuItemImage* pineappleItem = [CCMenuItemImage itemWithNormalSprite:pineappleSprite selectedSprite:pineappleSpriteSelected target:self selector:@selector(createPineapple:)];
        
        CCSprite* ropeSprite = [CCSprite spriteWithFile:@"ropeitem.png"];
        CCSprite* ropeSpriteSelected = [CCSprite spriteWithFile:@"ropeitem.png"];
        CCSprite* ropeSprite3 = [CCSprite spriteWithFile:@"ropeitem.png"];
        ropeSpriteSelected.color = ccc3(100, 0, 0);
        ropeSprite3.color = ccc3(100, 100, 100);
        ropeItem = [CCMenuItemImage itemWithNormalSprite:ropeSprite selectedSprite:ropeSpriteSelected disabledSprite:ropeSprite3 target:self selector:@selector(createRope:)];
        
        menu = [CCMenu menuWithItems: pineappleItem, ropeItem, nil];
        background = [CCSprite spriteWithFile:@"menu.png"];
        [background addChild:menu z:150];
        [self addChild:background];
        [parent addChild:self z:1000];
        [self setMenuEnabled:NO];
    }
    return self;
}

-(void)setPopupPosition:(CGPoint)position {
    tapPosition = position;
    
    // load defaultBackground and use its size to determine whether the popup still fits there
    CCSprite* defaultBackground = [CCSprite spriteWithFile:@"menu.png"];
    CGSize defaultBackgroundSize = defaultBackground.contentSize;
    float contentScaleFactor = [CCDirector sharedDirector].contentScaleFactor;
    float padding = defaultBackgroundSize.width*0.1f*contentScaleFactor;
    [menu alignItemsHorizontallyWithPadding:padding];
    
    CGPoint anchorPoint = CGPointMake(0.5f, 0.0f);
    CGPoint menuPosition = CGPointMake(defaultBackgroundSize.width/2, defaultBackgroundSize.height*0.7f);
    
	// Menu horizontal alignment
    CGSize winSize = [CCDirector sharedDirector].winSize;
    NSString* horizontalAlignment;
    if (position.x < defaultBackgroundSize.width/2) {
        // left
        horizontalAlignment = @"left";
        anchorPoint.x = 0.0f;
        [menu alignItemsVerticallyWithPadding:padding];
        menuPosition.x = defaultBackgroundSize.height * 0.7f;
        menuPosition.y = defaultBackgroundSize.width * 0.5f;
    } else if (winSize.width-position.x < defaultBackgroundSize.width/2) {
        // right
        horizontalAlignment = @"right";
        anchorPoint.x = 1.0f;
        [menu alignItemsVerticallyWithPadding:padding];
        menuPosition.x = defaultBackgroundSize.height * 0.3f;
        menuPosition.y = defaultBackgroundSize.width * 0.5f;
    } else {
        // center
        horizontalAlignment = @"center";
        [menu alignItemsHorizontallyWithPadding:padding];
    }
    // Menu vertical alignment
    NSString* verticalAlignment;
    if (position.y < defaultBackgroundSize.height) {
        // bottom
        verticalAlignment = @"bottom";
        if ([horizontalAlignment isEqualToString:@"center"]) {
            anchorPoint.y = 0.0f;
        } else {
            anchorPoint.y = 0.25f;
        }
    } else if (winSize.height - position.y < defaultBackgroundSize.height) {
        // top
        verticalAlignment = @"top";
        if ([horizontalAlignment isEqualToString:@"center"]) {
            anchorPoint.y = 1.0f;
            menuPosition.y = defaultBackgroundSize.height*0.3f;
        } else {
            anchorPoint.y = 0.75f;
        }
    } else {
        // middle
        verticalAlignment = @"middle";
        if ([horizontalAlignment isEqualToString:@"center"]) {
            anchorPoint.y = 0.0f;
        } else {
            anchorPoint.y = 0.5f;
        }
    }
    // Draw the menu
    NSString* filename = [NSString stringWithFormat: @"menu-%@-%@.png", verticalAlignment, horizontalAlignment];
    CCTexture2D* tex = [[CCTextureCache sharedTextureCache] addImage:filename];
    if (!tex) {
        tex = [[CCTextureCache sharedTextureCache] addImage:@"menu.png"];
    }
    [background setTexture:tex];
    [background setTextureRect: CGRectMake(0, 0, tex.contentSize.width, tex.contentSize.height)];
    
    background.anchorPoint = anchorPoint;
    background.position = position;
    background.opacity = menu.opacity;
    
    menu.position = menuPosition;
}

-(BOOL)isEnabled {
    return isEnabled;
}

-(void)setMenuEnabled:(BOOL)enable {
    for (CCMenuItem* item in menu.children) {
        item.isEnabled = enable;
    }
    isEnabled = enable;
    int opacity;
    if (enable) {
        opacity = 255;
    } else {
        opacity = 0;
    }
    background.opacity = opacity;
    menu.opacity = opacity;
}

-(void)setRopeItemEnabled:(BOOL)enabled {
    ropeItem.isEnabled = enabled;
}

-(void)createPineapple:(id)sender {
    [self.delegate createPineappleAt:tapPosition];
}

-(void)createRope:(id)sender {
    [self.delegate createRopeAt:tapPosition];
}

@end
