//
//  RopeSprite.m
//  CutTheVerlet
//
//  Created by Fahim Farook on 7/4/13.
//  Copyright (c) 2013 OpenTheJob. All rights reserved.
//

#import "RopeSprite.h"
#import "CoordinateHelper.h"

@interface RopeSprite () {
    CCSprite* ropeSprite;
    RopeModel* ropeModel;
    anchorType selectedAnchor;
}

@end

@implementation RopeSprite

-(id)initWithParent:(CCNode*)parent andRopeModel:(RopeModel*)aRopeModel {
    self = [super init];
    if (self) {
        ropeModel = aRopeModel;
        
        ropeSprite = [CCSprite spriteWithFile:@"rope_texture.png"];
        ccTexParams params = {GL_LINEAR,GL_LINEAR,GL_REPEAT,GL_CLAMP_TO_EDGE};
        [ropeSprite.texture setTexParameters:&params];
        
        [self updateRope];
        [parent addChild:ropeSprite];
    }
    return self;
}

-(void)updateRope {
    CGPoint anchorA = [CoordinateHelper levelPositionToScreenPosition:ropeModel.anchorA];
    CGPoint anchorB = [CoordinateHelper levelPositionToScreenPosition:ropeModel.anchorB];
    float distance = ccpDistance(anchorA, anchorB);
    CGPoint stickVector = ccpSub(ccp(anchorA.x,anchorA.y),ccp(anchorB.x,anchorB.y));
    float angle = ccpToAngle(stickVector);
    ropeSprite.textureRect = CGRectMake(0, 0, distance, ropeSprite.texture.pixelsHigh);
    ropeSprite.position = ccpMidpoint(anchorA, anchorB);
    ropeSprite.rotation = -1 * CC_RADIANS_TO_DEGREES(angle);
}

-(void)setAnchorA:(CGPoint)theAnchorA {
    ropeModel.anchorA = [CoordinateHelper screenPositionToLevelPosition:theAnchorA];
    [self updateRope];
}

-(void)setAnchorB:(CGPoint)theAnchorB {
    ropeModel.anchorB = [CoordinateHelper screenPositionToLevelPosition:theAnchorB];
    [self updateRope];
}

-(int)getID {
    return ropeModel.id;
}

-(CGPoint)getAnchorA {
    return [CoordinateHelper levelPositionToScreenPosition:ropeModel.anchorA];
}

-(CGPoint)getAnchorB {
    return [CoordinateHelper levelPositionToScreenPosition:ropeModel.anchorB];
}

-(int)getBodyAID {
    return ropeModel.bodyAID;
}

-(int)getBodyBID {
    return ropeModel.bodyBID;
}

-(float)distanceToPoint:(CGPoint)position {
    CGPoint anchorA = [CoordinateHelper levelPositionToScreenPosition:ropeModel.anchorA];
    CGPoint anchorB = [CoordinateHelper levelPositionToScreenPosition:ropeModel.anchorB];
    float deltaX = anchorA.x-anchorB.x;
    float deltaY = anchorA.y-anchorB.y;
    float distance;
    if (deltaX == 0) {
        distance = fabs(anchorA.x - position.x);
    } else if (deltaY == 0) {
        distance = fabs(anchorA.y - position.y);
    } else {
        // calculate slope by dividing y-coordinate distance with x-coordinate distance
        float slope = deltaY/deltaX;
        // calculate y-intercept of rope t = y-m*x
        float yIntercept = anchorA.y - anchorA.x * slope;
        // construct line that is orthographic to rope and goes through point, we want to calculate distance for
        float slopeOrthogonal = -1/slope;
        // position.y = slopeOrthogonal * position.x + yInterceptOrthogonal => solve for yInterceptOrthogonal
        float yInterceptOrthogonal = position.y - slopeOrthogonal * position.x;
        // calculate interception between rope and orthogonal line
        float x = (yInterceptOrthogonal - yIntercept) / (slope - slopeOrthogonal);
        float y = slope * x + yIntercept;
        
        distance = ccpDistance(position, CGPointMake(x, y));
    }
    return distance;
}

-(CGRect)getBoundingBox {
    return ropeSprite.boundingBox;
}

-(void)cleanupSprite {
    [ropeSprite removeFromParentAndCleanup:YES];
}

-(void)setSelectedAnchorType:(anchorType)type {
    selectedAnchor = type;
}

-(anchorType)getSelectedAnchorType {
    return selectedAnchor;
}

-(CGPoint)getSelectedAnchor {
    if (selectedAnchor == kAnchorA) {
        return [CoordinateHelper levelPositionToScreenPosition:ropeModel.anchorA];
    }
    if (selectedAnchor == kAnchorB) {
        return [CoordinateHelper levelPositionToScreenPosition:ropeModel.anchorB];
    }
    return CGPointMake(-1, -1);
}

-(void)moveSelectedAnchorTo:(CGPoint)position {
    if (selectedAnchor == kAnchorA) {
        ropeModel.anchorA = [CoordinateHelper screenPositionToLevelPosition:position];
    }
    if (selectedAnchor == kAnchorB) {
        ropeModel.anchorB = [CoordinateHelper screenPositionToLevelPosition:position];
    }
    [self updateRope];
}

-(BOOL)isValidNewAnchorID:(int)newAnchorID {
    int unselectedAnchorID;
    if (selectedAnchor == kAnchorA) {
        unselectedAnchorID = ropeModel.bodyBID;
    } else {
        unselectedAnchorID = ropeModel.bodyAID;
    }
    if (newAnchorID == unselectedAnchorID) {
        return NO;
    }
    return YES;
}

@end
